@@ -31,5 +31,8 @@ class TestGetFernet:
         assert isinstance(fernet, Fernet)
 
     def test_error(self) -> None:
-        with temp_environ({_ENV_VAR: None}), raises(GetFernetError):
+        with (
+            temp_environ({_ENV_VAR: None}),
+            raises(GetFernetError, match="Environment variable 'FERNET_KEY' is None"),
+        ):
             _ = get_fernet()
