@@ -1,11 +1,12 @@
 from __future__ import annotations
 
+import datetime as dt
 from itertools import pairwise
 from pathlib import Path
 from re import search
 from typing import TYPE_CHECKING, Any, cast
 
-from hypothesis import Phase, assume, given, settings
+from hypothesis import assume, given, settings
 from hypothesis.errors import InvalidArgument
 from hypothesis.extra.numpy import array_shapes
 from hypothesis.strategies import (
@@ -46,6 +47,7 @@ from utilities.hypothesis import (
     assume_does_not_raise,
     bool_arrays,
     date_deltas,
+    date_periods,
     date_time_deltas,
     dates,
     draw2,
@@ -57,6 +59,8 @@ from utilities.hypothesis import (
     hashables,
     import_froms,
     imports,
+    int8s,
+    int16s,
     int32s,
     int64s,
     int_arrays,
@@ -65,7 +69,7 @@ from utilities.hypothesis import (
     numbers,
     pairs,
     paths,
-    plain_datetimes,
+    plain_date_times,
     random_states,
     sentinels,
     sets_fixed_length,
@@ -82,6 +86,7 @@ from utilities.hypothesis import (
     text_digits,
     text_printable,
     time_deltas,
+    time_periods,
     times,
     triples,
     uint8s,
@@ -91,13 +96,16 @@ from utilities.hypothesis import (
     urls,
     versions,
     year_months,
-    zoned_datetimes,
+    zoned_date_time_periods,
+    zoned_date_times,
 )
 from utilities.iterables import one
 from utilities.libcst import parse_import
 from utilities.math import (
     MAX_FLOAT32,
     MAX_FLOAT64,
+    MAX_INT8,
+    MAX_INT16,
     MAX_INT32,
     MAX_INT64,
     MAX_UINT8,
@@ -106,6 +114,8 @@ from utilities.math import (
     MAX_UINT64,
     MIN_FLOAT32,
     MIN_FLOAT64,
+    MIN_INT8,
+    MIN_INT16,
     MIN_INT32,
     MIN_INT64,
     MIN_UINT8,
@@ -115,20 +125,24 @@ from utilities.math import (
 )
 from utilities.os import temp_environ
 from utilities.platform import maybe_yield_lower_case
-from utilities.sentinel import Sentinel
+from utilities.sentinel import is_sentinel
 from utilities.version import Version
 from utilities.whenever import (
     DATE_TWO_DIGIT_YEAR_MAX,
     DATE_TWO_DIGIT_YEAR_MIN,
+    DatePeriod,
+    TimePeriod,
+    ZonedDateTimePeriod,
     to_days,
     to_nanoseconds,
+    to_py_time_delta,
 )
 
 if TYPE_CHECKING:
-    from collections.abc import Iterable
     from collections.abc import Set as AbstractSet
     from zoneinfo import ZoneInfo
 
+    from utilities.sentinel import Sentinel
     from utilities.tempfile import TemporaryDirectory
     from utilities.types import Number
 
@@ -186,6 +200,7 @@ class TestDateDeltas:
                 date_deltas(min_value=min_value, max_value=max_value, parsable=parsable)
             )
         assert isinstance(delta, DateDelta)
+        assert isinstance(to_py_time_delta(delta), dt.timedelta)
         days = to_days(delta)
         if min_value is not None:
             assert days >= to_days(min_value)
@@ -195,15 +210,32 @@ class TestDateDeltas:
             assert DateDelta.parse_common_iso(delta.format_common_iso()) == delta
 
 
+class TestDatePeriods:
+    @given(data=data())
+    def test_main(self, *, data: DataObject) -> None:
+        min_value = data.draw(dates() | none())
+        max_value = data.draw(dates() | none())
+        with assume_does_not_raise(InvalidArgument):
+            period = data.draw(date_periods(min_value=min_value, max_value=max_value))
+        assert isinstance(period, DatePeriod)
+        if min_value is not None:
+            assert period.start >= min_value
+        if max_value is not None:
+            assert period.end <= max_value
+
+
 class TestDateTimeDeltas:
-    @given(data=data(), parsable=booleans())
-    def test_main(self, *, data: DataObject, parsable: bool) -> None:
+    @given(data=data(), parsable=booleans(), nativable=booleans())
+    def test_main(self, *, data: DataObject, parsable: bool, nativable: bool) -> None:
         min_value = data.draw(date_time_deltas() | none())
         max_value = data.draw(date_time_deltas() | none())
         with assume_does_not_raise(InvalidArgument):
             delta = data.draw(
                 date_time_deltas(
-                    min_value=min_value, max_value=max_value, parsable=parsable
+                    min_value=min_value,
+                    max_value=max_value,
+                    parsable=parsable,
+                    nativable=nativable,
                 )
             )
         assert isinstance(delta, DateTimeDelta)
@@ -214,6 +246,8 @@ class TestDateTimeDeltas:
             assert nanos <= to_nanoseconds(max_value)
         if parsable:
             assert DateTimeDelta.parse_common_iso(delta.format_common_iso()) == delta
+        if nativable:
+            assert isinstance(to_py_time_delta(delta), dt.timedelta)
 
 
 class TestDates:
@@ -347,17 +381,27 @@ class TestDraw2:
 class TestFloat32s:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        min_value, max_value = data.draw(pairs(float32s(), sorted=True))
-        x = data.draw(float32s(min_value=min_value, max_value=max_value))
-        assert max(min_value, MIN_FLOAT32) <= x <= min(max_value, MAX_FLOAT32)
+        min_value, max_value = data.draw(pairs(float32s() | none()))
+        with assume_does_not_raise(InvalidArgument):
+            x = data.draw(float32s(min_value=min_value, max_value=max_value))
+        assert MIN_FLOAT32 <= x <= MAX_FLOAT32
+        if min_value is not None:
+            assert x >= min_value
+        if max_value is not None:
+            assert x <= max_value
 
 
 class TestFloat64s:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        min_value, max_value = data.draw(pairs(float64s(), sorted=True))
-        x = data.draw(float64s(min_value=min_value, max_value=max_value))
-        assert max(min_value, MIN_FLOAT64) <= x <= min(max_value, MAX_FLOAT64)
+        min_value, max_value = data.draw(pairs(float64s() | none()))
+        with assume_does_not_raise(InvalidArgument):
+            x = data.draw(float64s(min_value=min_value, max_value=max_value))
+        assert MIN_FLOAT64 <= x <= MAX_FLOAT64
+        if min_value is not None:
+            assert x >= min_value
+        if max_value is not None:
+            assert x <= max_value
 
 
 class TestFloatArrays:
@@ -574,20 +618,56 @@ class TestIntArrays:
             assert len(set(flat)) == len(flat)
 
 
+class TestInt8s:
+    @given(data=data())
+    def test_main(self, *, data: DataObject) -> None:
+        min_value, max_value = data.draw(pairs(int8s() | none()))
+        with assume_does_not_raise(InvalidArgument):
+            x = data.draw(int8s(min_value=min_value, max_value=max_value))
+        assert MIN_INT8 <= x <= MAX_INT8
+        if min_value is not None:
+            assert x >= min_value
+        if max_value is not None:
+            assert x <= max_value
+
+
+class TestInt16s:
+    @given(data=data())
+    def test_main(self, *, data: DataObject) -> None:
+        min_value, max_value = data.draw(pairs(int16s() | none()))
+        with assume_does_not_raise(InvalidArgument):
+            x = data.draw(int16s(min_value=min_value, max_value=max_value))
+        assert MIN_INT16 <= x <= MAX_INT16
+        if min_value is not None:
+            assert x >= min_value
+        if max_value is not None:
+            assert x <= max_value
+
+
 class TestInt32s:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        min_value, max_value = data.draw(pairs(int32s(), sorted=True))
-        x = data.draw(int32s(min_value=min_value, max_value=max_value))
-        assert max(min_value, MIN_INT32) <= x <= min(max_value, MAX_INT32)
+        min_value, max_value = data.draw(pairs(int32s() | none()))
+        with assume_does_not_raise(InvalidArgument):
+            x = data.draw(int32s(min_value=min_value, max_value=max_value))
+        assert MIN_INT32 <= x <= MAX_INT32
+        if min_value is not None:
+            assert x >= min_value
+        if max_value is not None:
+            assert x <= max_value
 
 
 class TestInt64s:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        min_value, max_value = data.draw(pairs(int64s(), sorted=True))
-        x = data.draw(int64s(min_value=min_value, max_value=max_value))
-        assert max(min_value, MIN_INT64) <= x <= min(max_value, MAX_INT64)
+        min_value, max_value = data.draw(pairs(int64s() | none()))
+        with assume_does_not_raise(InvalidArgument):
+            x = data.draw(int64s(min_value=min_value, max_value=max_value))
+        assert MIN_INT64 <= x <= MAX_INT64
+        if min_value is not None:
+            assert x >= min_value
+        if max_value is not None:
+            assert x <= max_value
 
 
 class TestListsFixedLength:
@@ -655,8 +735,8 @@ class TestPairs:
 class TestPaths:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        min_depth = data.draw(integers(0, 5) | none())
-        max_depth = data.draw(integers(0, 5) | none())
+        min_depth = data.draw(integers(0, 3) | none())
+        max_depth = data.draw(integers(0, 3) | none())
         with assume_does_not_raise(InvalidArgument):
             path = data.draw(paths(min_depth=min_depth, max_depth=max_depth))
         assert isinstance(path, Path)
@@ -671,11 +751,11 @@ class TestPaths:
 class TestPlainDateTimes:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        min_value = data.draw(plain_datetimes() | none())
-        max_value = data.draw(plain_datetimes() | none())
+        min_value = data.draw(plain_date_times() | none())
+        max_value = data.draw(plain_date_times() | none())
         with assume_does_not_raise(InvalidArgument):
             datetime = data.draw(
-                plain_datetimes(min_value=min_value, max_value=max_value)
+                plain_date_times(min_value=min_value, max_value=max_value)
             )
         assert isinstance(datetime, PlainDateTime)
         assert PlainDateTime.parse_common_iso(datetime.format_common_iso()) == datetime
@@ -707,7 +787,7 @@ class TestSentinels:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
         sentinel = data.draw(sentinels())
-        assert isinstance(sentinel, Sentinel)
+        assert is_sentinel(sentinel)
 
 
 class TestSetsFixedLength:
@@ -722,13 +802,11 @@ class TestSetupHypothesisProfiles:
     def test_main(self) -> None:
         setup_hypothesis_profiles()
         curr = settings()
-        assert Phase.shrink in cast("Iterable[Phase]", curr.phases)
         assert curr.max_examples in {10, 100, 1000}
 
     def test_no_shrink(self) -> None:
         with temp_environ({"HYPOTHESIS_NO_SHRINK": "1"}):
             setup_hypothesis_profiles()
-        assert Phase.shrink not in cast("Iterable[Phase]", settings().phases)
 
     @given(max_examples=integers(1, 100))
     def test_max_examples(self, *, max_examples: int) -> None:
@@ -920,12 +998,27 @@ class TestTimeDeltas:
             delta = data.draw(time_deltas(min_value=min_value, max_value=max_value))
         assert isinstance(delta, TimeDelta)
         assert TimeDelta.parse_common_iso(delta.format_common_iso()) == delta
+        assert isinstance(to_py_time_delta(delta), dt.timedelta)
         if min_value is not None:
             assert delta >= min_value
         if max_value is not None:
             assert delta <= max_value
 
 
+class TestTimePeriods:
+    @given(data=data())
+    def test_main(self, *, data: DataObject) -> None:
+        min_value = data.draw(times() | none())
+        max_value = data.draw(times() | none())
+        with assume_does_not_raise(InvalidArgument):
+            period = data.draw(time_periods(min_value=min_value, max_value=max_value))
+        assert isinstance(period, TimePeriod)
+        if min_value is not None:
+            assert period.start >= min_value
+        if max_value is not None:
+            assert period.end <= max_value
+
+
 class TestTimes:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
@@ -958,33 +1051,53 @@ class TestTriples:
 class TestUInt8s:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        min_value, max_value = data.draw(pairs(uint8s(), sorted=True))
-        x = data.draw(uint8s(min_value=min_value, max_value=max_value))
-        assert max(min_value, MIN_UINT8) <= x <= min(max_value, MAX_UINT8)
+        min_value, max_value = data.draw(pairs(uint8s() | none()))
+        with assume_does_not_raise(InvalidArgument):
+            x = data.draw(uint8s(min_value=min_value, max_value=max_value))
+        assert MIN_UINT8 <= x <= MAX_UINT8
+        if min_value is not None:
+            assert x >= min_value
+        if max_value is not None:
+            assert x <= max_value
 
 
 class TestUInt16s:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        min_value, max_value = data.draw(pairs(uint16s(), sorted=True))
-        x = data.draw(uint16s(min_value=min_value, max_value=max_value))
-        assert max(min_value, MIN_UINT16) <= x <= min(max_value, MAX_UINT16)
+        min_value, max_value = data.draw(pairs(uint16s() | none()))
+        with assume_does_not_raise(InvalidArgument):
+            x = data.draw(uint16s(min_value=min_value, max_value=max_value))
+        assert MIN_UINT16 <= x <= MAX_UINT16
+        if min_value is not None:
+            assert x >= min_value
+        if max_value is not None:
+            assert x <= max_value
 
 
 class TestUInt32s:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        min_value, max_value = data.draw(pairs(uint32s(), sorted=True))
-        x = data.draw(uint32s(min_value=min_value, max_value=max_value))
-        assert max(min_value, MIN_UINT32) <= x <= min(max_value, MAX_UINT32)
+        min_value, max_value = data.draw(pairs(uint32s() | none()))
+        with assume_does_not_raise(InvalidArgument):
+            x = data.draw(uint32s(min_value=min_value, max_value=max_value))
+        assert MIN_UINT32 <= x <= MAX_UINT32
+        if min_value is not None:
+            assert x >= min_value
+        if max_value is not None:
+            assert x <= max_value
 
 
 class TestUInt64s:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        min_value, max_value = data.draw(pairs(uint64s(), sorted=True))
-        x = data.draw(uint64s(min_value=min_value, max_value=max_value))
-        assert max(min_value, MIN_UINT64) <= x <= min(max_value, MAX_UINT64)
+        min_value, max_value = data.draw(pairs(uint64s() | none()))
+        with assume_does_not_raise(InvalidArgument):
+            x = data.draw(uint64s(min_value=min_value, max_value=max_value))
+        assert MIN_UINT64 <= x <= MAX_UINT64
+        if min_value is not None:
+            assert x >= min_value
+        if max_value is not None:
+            assert x <= max_value
 
 
 class TestURLs:
@@ -1063,14 +1176,33 @@ class TestYearMonths:
             assert year_month.on_day(28) <= DATE_TWO_DIGIT_YEAR_MAX
 
 
+class TestZonedDateTimePeriods:
+    @given(data=data(), time_zone=timezones())
+    def test_main(self, *, data: DataObject, time_zone: ZoneInfo) -> None:
+        min_value = data.draw(zoned_date_times() | none())
+        max_value = data.draw(zoned_date_times() | none())
+        with assume_does_not_raise(InvalidArgument):
+            period = data.draw(
+                zoned_date_time_periods(
+                    min_value=min_value, max_value=max_value, time_zone=time_zone
+                )
+            )
+        assert isinstance(period, ZonedDateTimePeriod)
+        assert period.time_zone is time_zone
+        if min_value is not None:
+            assert period.start >= min_value
+        if max_value is not None:
+            assert period.end <= max_value
+
+
 class TestZonedDateTimes:
     @given(data=data(), time_zone=timezones())
     def test_main(self, *, data: DataObject, time_zone: ZoneInfo) -> None:
-        min_value = data.draw(zoned_datetimes() | none())
-        max_value = data.draw(zoned_datetimes() | none())
+        min_value = data.draw(zoned_date_times() | none())
+        max_value = data.draw(zoned_date_times() | none())
         with assume_does_not_raise(InvalidArgument):
             datetime = data.draw(
-                zoned_datetimes(
+                zoned_date_times(
                     min_value=min_value, max_value=max_value, time_zone=time_zone
                 )
             )
@@ -1087,5 +1219,5 @@ class TestZonedDateTimes:
     def test_examples(self, *, data: DataObject, time_zone: ZoneInfo) -> None:
         with assume_does_not_raise(TimeZoneNotFoundError):
             max_value = ZonedDateTime(1, 1, 2, tz=time_zone.key)
-        datetime = data.draw(zoned_datetimes(max_value=max_value, time_zone=time_zone))
+        datetime = data.draw(zoned_date_times(max_value=max_value, time_zone=time_zone))
         _ = datetime.py_datetime()
