@@ -22,6 +22,7 @@ from utilities.os import (
     get_cpu_use,
     get_env_var,
     is_debug,
+    is_pytest,
     temp_environ,
 )
 from utilities.pytest import skipif_windows
@@ -124,6 +125,11 @@ class TestIsDebug:
         assert isinstance(result, bool)
 
 
+class TestIsPytest:
+    def test_main(self) -> None:
+        assert is_pytest()
+
+
 class TestTempEnviron:
     @given(key=text.map(_prefix), value=text)
     def test_set(self, *, key: str, value: str) -> None:
