@@ -31,7 +31,7 @@ from typing import (
     override,
 )
 
-from whenever import PlainDateTime, ZonedDateTime
+from whenever import ZonedDateTime
 
 from utilities.atomicwrites import move_many
 from utilities.dataclasses import replace_non_sentinel
@@ -45,16 +45,15 @@ from utilities.re import (
     extract_groups,
 )
 from utilities.sentinel import Sentinel, sentinel
-from utilities.tzlocal import LOCAL_TIME_ZONE_NAME
 from utilities.whenever import (
     WheneverLogRecord,
-    format_compact,
     get_now_local,
+    parse_plain_local,
     to_local_plain,
 )
 
 if TYPE_CHECKING:
-    from collections.abc import Callable, Iterable, Mapping
+    from collections.abc import Iterable, Mapping
     from datetime import time
     from logging import _FilterType
 
@@ -151,42 +150,6 @@ def basic_config(
 ##
 
 
-def filter_for_key(
-    key: str, /, *, default: bool = False
-) -> Callable[[LogRecord], bool]:
-    """Make a filter for a given attribute."""
-    if (key in _FILTER_FOR_KEY_BLACKLIST) or key.startswith("__"):
-        raise FilterForKeyError(key=key)
-
-    def filter_(record: LogRecord, /) -> bool:
-        try:
-            value = getattr(record, key)
-        except AttributeError:
-            return default
-        return bool(value)
-
-    return filter_
-
-
-# fmt: off
-_FILTER_FOR_KEY_BLACKLIST = {
-    "args", "created", "exc_info", "exc_text", "filename", "funcName", "getMessage", "levelname", "levelno", "lineno", "module", "msecs", "msg", "name", "pathname", "process", "processName", "relativeCreated", "stack_info", "taskName", "thread", "threadName"
-}
-# fmt: on
-
-
-@dataclass(kw_only=True, slots=True)
-class FilterForKeyError(Exception):
-    key: str
-
-    @override
-    def __str__(self) -> str:
-        return f"Invalid key: {self.key!r}"
-
-
-##
-
-
 def get_format_str(*, prefix: str | None = None, hostname: bool = False) -> str:
     """Generate a format string."""
     parts: list[str] = [
@@ -535,10 +498,8 @@ class _RotatingLogFile:
                 stem=stem,
                 suffix=suffix,
                 index=int(index),
-                start=PlainDateTime.parse_common_iso(start).assume_tz(
-                    LOCAL_TIME_ZONE_NAME
-                ),
-                end=PlainDateTime.parse_common_iso(end).assume_tz(LOCAL_TIME_ZONE_NAME),
+                start=parse_plain_local(start),
+                end=parse_plain_local(end),
             )
         try:
             index, end = extract_groups(patterns.pattern2, path.name)
@@ -550,7 +511,7 @@ class _RotatingLogFile:
                 stem=stem,
                 suffix=suffix,
                 index=int(index),
-                end=PlainDateTime.parse_common_iso(end).assume_tz(LOCAL_TIME_ZONE_NAME),
+                end=parse_plain_local(end),
             )
         try:
             index = extract_group(patterns.pattern1, path.name)
@@ -571,9 +532,9 @@ class _RotatingLogFile:
             case int() as index, None, None:
                 tail = str(index)
             case int() as index, None, ZonedDateTime() as end:
-                tail = f"{index}__{format_compact(to_local_plain(end))}"
+                tail = f"{index}__{to_local_plain(end)}"
             case int() as index, ZonedDateTime() as start, ZonedDateTime() as end:
-                tail = f"{index}__{format_compact(to_local_plain(start))}__{format_compact(to_local_plain(end))}"
+                tail = f"{index}__{to_local_plain(start)}__{to_local_plain(end)}"
             case _:  # pragma: no cover
                 raise ImpossibleCaseError(
                     case=[f"{self.index=}", f"{self.start=}", f"{self.end=}"]
@@ -626,12 +587,10 @@ def to_logger(logger: LoggerLike | None = None, /) -> Logger:
 
 
 __all__ = [
-    "FilterForKeyError",
     "GetLoggingLevelNumberError",
     "SizeAndTimeRotatingFileHandler",
     "add_filters",
     "basic_config",
-    "filter_for_key",
     "get_format_str",
     "get_logging_level_number",
     "setup_logging",
