@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-import datetime as dt
 import enum
 from collections.abc import Callable, Iterator, Sequence
 from collections.abc import Set as AbstractSet
@@ -15,11 +14,12 @@ from uuid import UUID
 from zoneinfo import ZoneInfo
 
 import polars as pl
+import whenever
 from polars import (
     Boolean,
     DataFrame,
-    Date,
     Datetime,
+    Duration,
     Expr,
     Float64,
     Int64,
@@ -43,16 +43,16 @@ from polars import (
 from polars._typing import PolarsDataType
 from polars.datatypes import DataType, DataTypeClass
 from polars.exceptions import (
-    ColumnNotFoundError,
+    ColumnNotFoundError,  # pyright: ignore[reportAttributeAccessIssue]
     NoRowsReturnedError,
-    OutOfBoundsError,
+    OutOfBoundsError,  # pyright: ignore[reportAttributeAccessIssue]
     PolarsInefficientMapWarning,
 )
 from polars.schema import Schema
 from polars.testing import assert_frame_equal, assert_series_equal
-from whenever import ZonedDateTime
+from whenever import DateDelta, DateTimeDelta, PlainDateTime, TimeDelta, ZonedDateTime
 
-from utilities.dataclasses import _YieldFieldsInstance, yield_fields
+from utilities.dataclasses import yield_fields
 from utilities.errors import ImpossibleCaseError
 from utilities.functions import (
     EnsureIntError,
@@ -93,14 +93,18 @@ from utilities.typing import (
     get_args,
     get_type_hints,
     is_frozenset_type,
-    is_instance_gen,
     is_list_type,
     is_literal_type,
     is_optional_type,
     is_set_type,
-    is_union_type,
 )
 from utilities.warnings import suppress_warnings
+from utilities.whenever import (
+    DatePeriod,
+    TimePeriod,
+    ZonedDateTimePeriod,
+    to_py_time_delta,
+)
 from utilities.zoneinfo import UTC, ensure_time_zone, get_time_zone_name
 
 if TYPE_CHECKING:
@@ -131,6 +135,10 @@ DatetimeTokyo = Datetime(time_zone="Asia/Tokyo")
 DatetimeUSCentral = Datetime(time_zone="US/Central")
 DatetimeUSEastern = Datetime(time_zone="US/Eastern")
 DatetimeUTC = Datetime(time_zone="UTC")
+DatePeriodDType = Struct({"start": pl.Date, "end": pl.Date})
+TimePeriodDType = Struct({"start": pl.Time, "end": pl.Time})
+
+
 _FINITE_EWM_MIN_WEIGHT = 0.9999
 
 
@@ -453,29 +461,6 @@ class BooleanValueCountsError(Exception):
 ##
 
 
-@overload
-def ceil_datetime(column: ExprLike, every: ExprLike, /) -> Expr: ...
-@overload
-def ceil_datetime(column: Series, every: ExprLike, /) -> Series: ...
-@overload
-def ceil_datetime(column: IntoExprColumn, every: ExprLike, /) -> ExprOrSeries: ...
-def ceil_datetime(column: IntoExprColumn, every: ExprLike, /) -> ExprOrSeries:
-    """Compute the `ceil` of a datetime column."""
-    column = ensure_expr_or_series(column)
-    rounded = column.dt.round(every)
-    ceil = (
-        when(column <= rounded)
-        .then(rounded)
-        .otherwise(column.dt.offset_by(every).dt.round(every))
-    )
-    if isinstance(column, Expr):
-        return ceil
-    return DataFrame().with_columns(ceil.alias(column.name))[column.name]
-
-
-##
-
-
 def check_polars_dataframe(
     df: DataFrame,
     /,
@@ -1051,16 +1036,41 @@ def dataclass_to_dataframe(
 
 
 def _dataclass_to_dataframe_cast(series: Series, /) -> Series:
-    if series.dtype == Object:
-        is_path = series.map_elements(make_isinstance(Path), return_dtype=Boolean).all()
-        is_uuid = series.map_elements(make_isinstance(UUID), return_dtype=Boolean).all()
-        if is_path or is_uuid:
-            with suppress_warnings(category=PolarsInefficientMapWarning):
-                return series.map_elements(str, return_dtype=String)
-        else:  # pragma: no cover
-            msg = f"{is_path=}, f{is_uuid=}"
-            raise NotImplementedError(msg)
-    return series
+    if series.dtype != Object:
+        return series
+    if series.map_elements(make_isinstance(whenever.Date), return_dtype=Boolean).all():
+        return series.map_elements(lambda x: x.py_date(), return_dtype=pl.Date)
+    if series.map_elements(make_isinstance(DateDelta), return_dtype=Boolean).all():
+        return series.map_elements(to_py_time_delta, return_dtype=Duration)
+    if series.map_elements(make_isinstance(DateTimeDelta), return_dtype=Boolean).all():
+        return series.map_elements(to_py_time_delta, return_dtype=Duration)
+    is_path = series.map_elements(make_isinstance(Path), return_dtype=Boolean).all()
+    is_uuid = series.map_elements(make_isinstance(UUID), return_dtype=Boolean).all()
+    if is_path or is_uuid:
+        with suppress_warnings(category=PolarsInefficientMapWarning):
+            return series.map_elements(str, return_dtype=String)
+    if series.map_elements(make_isinstance(whenever.Time), return_dtype=Boolean).all():
+        return series.map_elements(lambda x: x.py_time(), return_dtype=pl.Time)
+    if series.map_elements(make_isinstance(TimeDelta), return_dtype=Boolean).all():
+        return series.map_elements(to_py_time_delta, return_dtype=Duration)
+    if series.map_elements(make_isinstance(ZonedDateTime), return_dtype=Boolean).all():
+        return_dtype = zoned_date_time_dtype(time_zone=one({dt.tz for dt in series}))
+        return series.map_elements(lambda x: x.py_datetime(), return_dtype=return_dtype)
+    if series.map_elements(
+        lambda x: isinstance(x, dict) and (set(x) == {"start", "end"}),
+        return_dtype=Boolean,
+    ).all():
+        start = _dataclass_to_dataframe_cast(
+            series.map_elements(lambda x: x["start"], return_dtype=Object)
+        ).alias("start")
+        end = _dataclass_to_dataframe_cast(
+            series.map_elements(lambda x: x["end"], return_dtype=Object)
+        ).alias("end")
+        name = series.name
+        return concat_series(start, end).select(
+            struct(start=start, end=end).alias(name)
+        )[name]
+    raise NotImplementedError(series)  # pragma: no cover
 
 
 @dataclass(kw_only=True, slots=True)
@@ -1101,20 +1111,14 @@ def dataclass_to_schema(
     for field in yield_fields(
         obj, globalns=globalns, localns=localns, warn_name_errors=warn_name_errors
     ):
-        if is_dataclass_instance(field.value):
+        if is_dataclass_instance(field.value) and not (
+            isinstance(field.type_, type)
+            and issubclass(field.type_, (DatePeriod, TimePeriod, ZonedDateTimePeriod))
+        ):
             dtypes = dataclass_to_schema(
                 field.value, globalns=globalns, localns=localns
             )
             dtype = struct_dtype(**dtypes)
-        elif field.type_ is dt.datetime:
-            dtype = _dataclass_to_schema_datetime(field)
-        elif is_union_type(field.type_) and set(
-            get_args(field.type_, optional_drop_none=True)
-        ) == {dt.date, dt.datetime}:
-            if is_instance_gen(field.value, dt.date):
-                dtype = Date
-            else:
-                dtype = _dataclass_to_schema_datetime(field)
         else:
             dtype = _dataclass_to_schema_one(
                 field.type_, globalns=globalns, localns=localns
@@ -1123,14 +1127,6 @@ def dataclass_to_schema(
     return out
 
 
-def _dataclass_to_schema_datetime(
-    field: _YieldFieldsInstance[dt.datetime], /
-) -> PolarsDataType:
-    if field.value.tzinfo is None:
-        return Datetime
-    return zoned_datetime_dtype(time_zone=ensure_time_zone(field.value.tzinfo))
-
-
 def _dataclass_to_schema_one(
     obj: Any,
     /,
@@ -1138,20 +1134,35 @@ def _dataclass_to_schema_one(
     globalns: StrMapping | None = None,
     localns: StrMapping | None = None,
 ) -> PolarsDataType:
-    if obj is bool:
-        return Boolean
-    if obj is int:
-        return Int64
-    if obj is float:
-        return Float64
-    if obj is str:
-        return String
-    if obj is dt.date:
-        return Date
-    if obj in {Path, UUID}:
-        return Object
-    if isinstance(obj, type) and issubclass(obj, enum.Enum):
-        return pl.Enum([e.name for e in obj])
+    if isinstance(obj, type):
+        if issubclass(obj, bool):
+            return Boolean
+        if issubclass(obj, int):
+            return Int64
+        if issubclass(obj, float):
+            return Float64
+        if issubclass(obj, str):
+            return String
+        if issubclass(
+            obj,
+            (
+                DateDelta,
+                DatePeriod,
+                DateTimeDelta,
+                Path,
+                PlainDateTime,
+                TimeDelta,
+                TimePeriod,
+                UUID,
+                ZonedDateTime,
+                ZonedDateTimePeriod,
+                whenever.Date,
+                whenever.Time,
+            ),
+        ):
+            return Object
+        if issubclass(obj, enum.Enum):
+            return pl.Enum([e.name for e in obj])
     if is_dataclass_class(obj):
         out: dict[str, Any] = {}
         for field in yield_fields(obj, globalns=globalns, localns=localns):
@@ -1335,29 +1346,6 @@ class _FiniteEWMWeightsError(Exception):
 ##
 
 
-@overload
-def floor_datetime(column: ExprLike, every: ExprLike, /) -> Expr: ...
-@overload
-def floor_datetime(column: Series, every: ExprLike, /) -> Series: ...
-@overload
-def floor_datetime(column: IntoExprColumn, every: ExprLike, /) -> ExprOrSeries: ...
-def floor_datetime(column: IntoExprColumn, every: ExprLike, /) -> ExprOrSeries:
-    """Compute the `floor` of a datetime column."""
-    column = ensure_expr_or_series(column)
-    rounded = column.dt.round(every)
-    floor = (
-        when(column >= rounded)
-        .then(rounded)
-        .otherwise(column.dt.offset_by("-" + every).dt.round(every))
-    )
-    if isinstance(column, Expr):
-        return floor
-    return DataFrame().with_columns(floor.alias(column.name))[column.name]
-
-
-##
-
-
 def get_data_type_or_series_time_zone(
     dtype_or_series: PolarsDataType | Series, /
 ) -> ZoneInfo:
@@ -2383,7 +2371,8 @@ def round_to_float(
 ) -> ExprOrSeries:
     """Round a column to the nearest multiple of another float."""
     x = ensure_expr_or_series(x)
-    return (x / y).round(mode=mode) * y
+    z = (x / y).round(mode=mode) * y
+    return z.round(decimals=number_of_decimals(y) + 1)
 
 
 ##
@@ -2443,13 +2432,15 @@ def struct_from_dataclass(
 def _struct_from_dataclass_one(
     ann: Any, /, *, time_zone: TimeZoneLike | None = None
 ) -> PolarsDataType:
-    mapping = {bool: Boolean, dt.date: Date, float: Float64, int: Int64, str: String}
+    mapping = {
+        bool: Boolean,
+        whenever.Date: pl.Date,
+        float: Float64,
+        int: Int64,
+        str: String,
+    }
     with suppress(KeyError):
         return mapping[ann]
-    if ann is dt.datetime:
-        if time_zone is None:
-            raise _StructFromDataClassTimeZoneMissingError
-        return zoned_datetime_dtype(time_zone=time_zone)
     if is_dataclass_class(ann):
         return struct_from_dataclass(ann, time_zone=time_zone)
     if (isinstance(ann, type) and issubclass(ann, enum.Enum)) or (
@@ -2478,13 +2469,6 @@ class _StructFromDataClassNotADataclassError(StructFromDataClassError):
         return f"Object must be a dataclass; got {self.cls}"
 
 
-@dataclass(kw_only=True, slots=True)
-class _StructFromDataClassTimeZoneMissingError(StructFromDataClassError):
-    @override
-    def __str__(self) -> str:
-        return "Time-zone must be given"
-
-
 @dataclass(kw_only=True, slots=True)
 class _StructFromDataClassTypeError(StructFromDataClassError):
     ann: Any
@@ -2578,27 +2562,27 @@ def week_num(column: IntoExprColumn, /, *, start: WeekDay = "mon") -> ExprOrSeri
 ##
 
 
-def zoned_datetime_dtype(
+def zoned_date_time_dtype(
     *, time_unit: TimeUnit = "us", time_zone: TimeZoneLike = UTC
 ) -> Datetime:
-    """Create a zoned datetime data type."""
+    """Create a zoned date-time data type."""
     return Datetime(time_unit=time_unit, time_zone=get_time_zone_name(time_zone))
 
 
-def zoned_datetime_period_dtype(
+def zoned_date_time_period_dtype(
     *,
     time_unit: TimeUnit = "us",
     time_zone: TimeZoneLike | tuple[TimeZoneLike, TimeZoneLike] = UTC,
 ) -> Struct:
-    """Create a zoned datetime period data type."""
+    """Create a zoned date-time period data type."""
     match time_zone:
         case start, end:
             return struct_dtype(
-                start=zoned_datetime_dtype(time_unit=time_unit, time_zone=start),
-                end=zoned_datetime_dtype(time_unit=time_unit, time_zone=end),
+                start=zoned_date_time_dtype(time_unit=time_unit, time_zone=start),
+                end=zoned_date_time_dtype(time_unit=time_unit, time_zone=end),
             )
         case _:
-            dtype = zoned_datetime_dtype(time_unit=time_unit, time_zone=time_zone)
+            dtype = zoned_date_time_dtype(time_unit=time_unit, time_zone=time_zone)
             return struct_dtype(start=dtype, end=dtype)
 
 
@@ -2607,6 +2591,7 @@ __all__ = [
     "CheckPolarsDataFrameError",
     "ColumnsToDictError",
     "DataClassToDataFrameError",
+    "DatePeriodDType",
     "DatetimeHongKong",
     "DatetimeTokyo",
     "DatetimeUSCentral",
@@ -2624,6 +2609,7 @@ __all__ = [
     "IsNullStructSeriesError",
     "SetFirstRowAsColumnsError",
     "StructFromDataClassError",
+    "TimePeriodDType",
     "acf",
     "adjust_frequencies",
     "all_dataframe_columns",
@@ -2634,7 +2620,6 @@ __all__ = [
     "are_frames_equal",
     "bernoulli",
     "boolean_value_counts",
-    "ceil_datetime",
     "check_polars_dataframe",
     "choice",
     "collect_series",
@@ -2651,7 +2636,6 @@ __all__ = [
     "ensure_expr_or_series",
     "ensure_expr_or_series_many",
     "finite_ewm_mean",
-    "floor_datetime",
     "get_data_type_or_series_time_zone",
     "get_expr_name",
     "get_frequency_spectrum",
@@ -2687,6 +2671,6 @@ __all__ = [
     "unique_element",
     "write_dataframe",
     "write_series",
-    "zoned_datetime_dtype",
-    "zoned_datetime_period_dtype",
+    "zoned_date_time_dtype",
+    "zoned_date_time_period_dtype",
 ]
