@@ -18,6 +18,7 @@ from itertools import chain
 from math import floor
 from operator import ge, le
 from re import search
+from socket import gaierror
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -94,6 +95,7 @@ from utilities.iterables import (
     merge_str_mappings,
     one,
 )
+from utilities.os import is_pytest
 from utilities.reprlib import get_repr
 from utilities.text import secret_str, snake_case
 from utilities.types import (
@@ -127,7 +129,7 @@ def check_connect(engine: Engine, /) -> bool:
     try:
         with engine.connect() as conn:
             return bool(conn.execute(_SELECT).scalar_one())
-    except (OperationalError, ProgrammingError):
+    except (gaierror, ConnectionRefusedError, OperationalError, ProgrammingError):
         return False
 
 
@@ -138,7 +140,13 @@ async def check_connect_async(
     try:
         async with timeout_td(timeout), engine.connect() as conn:
             return bool((await conn.execute(_SELECT)).scalar_one())
-    except (OperationalError, ProgrammingError, TimeoutError):
+    except (
+        gaierror,
+        ConnectionRefusedError,
+        OperationalError,
+        ProgrammingError,
+        TimeoutError,
+    ):
         return False
 
 
@@ -969,8 +977,6 @@ async def yield_connection(
         async with timeout_td(timeout, error=error), engine.begin() as conn:
             yield conn
     except GeneratorExit:  # pragma: no cover
-        from utilities.pytest import is_pytest
-
         if not is_pytest():
             raise
         return
